// Persistence of Vision Ray Tracer Scene Description File
// File: SISray.pov
// Vers: 3.5
// Desc: Single Image Stereograms ala POV pigment functions. 
//       Based on C code by W.A. Steer Ph.D., fully described here: http://www.techmind.org/stereo/stech.html
// Date: Saturday, September 22, 2001 -- January 27, 2003 ( off and on )
// Auth: Barrett L. Bowen a.k.a. "Barry" or "Batronyx" ^"^

#include "colors.inc"
#default{finish{ ambient 1 }}

#declare iw = image_width;
#declare ih = image_height;
#declare dpi = 75;
#declare shift = dpi/32;
#declare obsDist=dpi*12;
#declare eyeSep=dpi*2.5;
#declare maxdepth = dpi*12;             
#declare maxsep=((eyeSep*maxdepth)/(maxdepth+obsDist));
#declare depthFac = 0.75;
#declare depthOffset = 0.25;

#declare Ptrn=function{
     pattern{ //crackle metric 2 form.75*x 
              // bozo //      scale .25
               granite scale 3  ////
          scale 0.15
          scale <maxsep,(iw/ih)*maxsep,1>
          //scallop_wave frequency 4
          warp{repeat x*maxsep}
     }
}

#declare H =
     function
     { pigment { image_map { sys "LogoDepthmap.bmp"  once}  
          translate y*-0.5
          scale <iw,ih,1>  
          translate <maxsep, ih/2, 0>}
     }

#declare featureZ=function(x,y,z){(1-(H(x,y,z).gray)*depthFac)}
#declare sep=function(x,y,z){ (eyeSep * featureZ(x,y,z))/( obsDist + featureZ(x,y,z))}
#declare Left = function(x,y,z){x - (sep(x,y,z)/2)} 
#declare Right = function(x,y,z){x + (sep(x,y,z)/2)}
     
#declare P1=function(x,y,z){    
     Ptrn(mod(x,maxsep),(y+(x/maxsep)*shift),z)//.gray
}

#declare FZ=function(x,y,z){  
     featureZ(x,y,z)
}

//Adjust Depthmap to the left a little
#declare FZL=function(x,y,z){  
     featureZ(x-(maxsep/2),y,z)
}

// Adjust Depthmp to the right a little.
#declare FZR=function(x,y,z){  
     featureZ(x+(maxsep/2),y,z)
}

//SLR == (Select Left Right)
#declare SLR = function(x,y,z) {
   select( (x <= (0.5*iw) + ((maxsep/2)  * FZR(x,y,z)) ) // 
          ,0
          ,select( (x >= (0.5*iw) + ((maxsep * 1.5) * FZL(x,y,z)) )
                 ,0
                 ,0
                 ,-1
           )
          ,1
   )
}

#declare LinkFactor = 3.2; // increase this if you get garbage at the edges. It slows the render time though.
#declare MaxCount = ceil(iw/(maxsep/LinkFactor))+1;
#debug str(MaxCount, 0, 3)
#declare Count = MaxCount;
#fopen LINKS "SISlinks.inc" write
#while(Count >= 0 )//MaxCount
   #write(LINKS, concat("#declare link",str(Count,0,0)," = function(x,y,z){\n"))
   #write(LINKS, "   select( SLR(x,y,z)\n") 
   #write(LINKS, "      ,select( (Left(x,y,z)=Right(x,y,z))\n")
   #write(LINKS, "            , 0                                 //never used here\n")      
   #if (Count = MaxCount)//0
      #write(LINKS, "            , (P1(x-(FZL(Right(x,y,z),y,z)*maxsep),y,z))           //False/Different \n")   
//      #write(LINKS, "            , (P1(Left(x,y,z)+(FZ(Left(x,y,z),y,z)*maxsep),y,z))           //True/Same \n")
      #write(LINKS, "            , (P1(x,y,z))           //True/Same \n")
   #else                               //    
      #write(LINKS, concat("            , link",str(Count+1,0,0),"(Right(x,y,z)-(FZL(Right(x,y,z),y,z)*maxsep),y,z)           //False/Different \n"))
      #write(LINKS, concat("            , link",str(Count+1,0,0),"(Left(x,y,z)-(FZL(Left(x,y,z),y,z)*maxsep),y,z)           //True/Same \n"))
   #end
   #write(LINKS, "      )\n")
   #write(LINKS, "      ,P1(x,y,z)\n")
   #write(LINKS, "      ,select( (Left(x,y,z)=Right(x,y,z))\n")
   #write(LINKS, "            , 0                                 //never used here\n")      
   #if (Count = MaxCount)//0
      #write(LINKS, "            , (P1(x+(FZR(Right(x,y,z),y,z)*maxsep),y,z))           //False/Different \n")   
//      #write(LINKS, "            , (P1(Left(x,y,z)-(FZ(Left(x,y,z),y,z)*maxsep),y,z))           //True/Same \n")
      #write(LINKS, "            , (P1(x,y,z))           //True/Same \n")
   #else                               //    
      #write(LINKS, concat("            , link",str(Count+1,0,0),"(Right(x,y,z)+(FZR(Right(x,y,z),y,z)*maxsep),y,z)           //False/Different \n"))
      #write(LINKS, concat("            , link",str(Count+1,0,0),"(Left(x,y,z)+(FZR(Left(x,y,z),y,z)*maxsep),y,z)           //True/Same \n"))
   #end
   #write(LINKS, "      )\n")
   #write(LINKS, "  )\n")
   #write(LINKS, "}\n\n")      
   #declare Count = Count -1;
#end   
#fclose LINKS
#include "SISlinks.inc"



#declare Pix=function(x,y,z){
   select( SLR(x,y,z)    
      ,select( (Left(x,y,z)=Right(x,y,z))
            , 0                                 //never used here
            , link0(Right(x,y,z)-(FZL(Right(x,y,z),y,z)*maxsep),y,z)          //False/Different
            , link0(Left(x,y,z)-(FZL(Left(x,y,z),y,z)*maxsep),y,z)           //True/Same 
      )
      ,P1(x,y,z)
      ,select( (Left(x,y,z)=Right(x,y,z))
            , 0                                 //never used here
            , link0(Right(x,y,z)+(FZR(Right(x,y,z),y,z)*maxsep),y,z)          //False/Different  
            , link0(Left(x,y,z)+(FZR(Left(x,y,z),y,z)*maxsep),y,z)           //True/Same 
      )
  )          
}
plane{-z, -1 
     pigment{function{Pix(x,y,z)}    
      //scallop_wave frequency 2
          color_map {
               [0 color rgb 0 ]
               [1 color rgb 1]
          }
     }
     finish{ambient 1}
     } 
       
camera{orthographic    //(maxsep*2)
      location <0.5*iw+maxsep,ih/2,-1>
      up y*ih
      right x*iw 
      look_at  <0.5*iw+maxsep,ih/2,0>      
}

        
          
               
                    
                         
                              
                                   
                                        
                                             
                                                  
                                                       
                                                            
                                                                 
                                                  
                                                       
                                                            
                                                                 
                                                                      
                                                                           
                                                                                
                                                                                     
                                                                                          
                                                                                               
                                                                                                    